@@ -5,6 +5,9 @@
 # standard imports
 import logging
 
+# local imports
+from .error import TokenExistsError
+
 logg = logging.getLogger(__file__)
 
 
@@ -110,6 +113,26 @@ class ContractRegistry:
     loaded = False
 
 
+    @staticmethod
+    def get_address(address):
+        """Return contract by address
+
+        :return: Contract if found 
+        :rtype: Contract or None
+        """
+        return ContractRegistry.addresses.get(address)
+
+
+    @staticmethod
+    def cache_token_count():
+        """Return number of tokens in registry cache
+
+        :return: Number of tokens
+        :rtype: int
+        """
+        return len(ContractRegistry.tokens)
+
+
     @staticmethod
     def get_token_by_symbol(symbol):
         """Look up token address by symbol
@@ -168,6 +191,14 @@ class ContractRegistry:
             converter = converters[0]
 
         t = Token(address, contract, typ, converter)
+
+        try:
+            ContractRegistry.addresses[t.address]
+            #ContractRegistry.get_token_by_symbol(t.symbol)
+            raise TokenExistsError('{} {}'.format(t.address, t.symbol))
+        except KeyError:
+            pass
+
         logg.info('adding token "{}" ({}) at {}'.format(t.symbol(), t.name(), address))
         ContractRegistry.tokens[t.symbol()] = t
         ContractRegistry.addresses[address] = t
@@ -200,6 +231,3 @@ class ContractRegistry:
         :rtype: dict
         """
         return ContractRegistry.contracts[contract_key].contract.abi
-
-
-
